; Haras: 
; 1KB minigame
; by Stefano Tognon
; (C) 2003 Ice Team
; version 1.00

; Well, this is a little dedicated game... name is obvious...
; This is my 7 game for C64 

; some restriction are applied for reducing space occupation:
; no self-made char
; up direction is not available
; monster walk only without using msb bit of x direction

      processor 6502

PUT_CHAR   = $E716    ; put a char in the screen
SCREEN     = $E544    ; clean the screen
SET_COOR   = $E50C    ; set x/y coord
GET_COOR   = $E513    ; get x/y coord
STRING     = $AB1E    ; put a string

SP0X        = $D000
SP0Y        = $D001
SP1X        = $D002
SP1Y        = $D003
SP2X        = $D004
SP2Y        = $D005
SP3X        = $D006
SP3Y        = $D007
SP4X        = $D008
SP4Y        = $D009
SP5X        = $D00A
SP5Y        = $D00B
SPX         = $D010
RASTER      = $D012
SPENABLE    = $D015
SPMULTIC    = $D01C
SPX_EXP     = $D01D
SP_SCOLL    = $D01E
BORDER_COL0 = $D020
BACK_COL0   = $D021
BACK_COL1   = $D022
BACK_COL2   = $D023
SP_MCOL0    = $D025
SP_MCOL1    = $D026
SP0_COL     = $D027
SP1_COL     = $D028
SP2_COL     = $D029

POSX       = $B2      ; address to the positions x
POSY       = $B4      ; address to the positions y
LEN        = $B6      ; len of positions
TAKE       = $B7      ; number of oil taken
DIR        = $B8      ; actual dir 0=dn 1=sx, FF=dx
DIRS       = $B9      ; store list of dir
TEMP       = $BD
DELAY      = $BE      ; delay for each frame
GO         = $BF      ; 1 = go with the game

MAXDELAY  = 4         ; frame delay
MAX_OIL   = (6*40-1)

RANDOM      = 54272+27 ; random number from osc 3

SC_GREEN    = 30    ; green color
SC_WHITE    =  5    ; white color
SP_SPACE    = 32    ; space
SP_OIL      = 46    ; oil to pick-up
SC_EA1      = 166   ; earth 1 
STEP        = 2     ; step of a monster


; the position of the oil pomp is controlled by:
;  3000 X [x1][x2][x3][x4][x5]       
;  4000 Y [y1][y2][y3][y4][y5]
;  5000 Dir direction of oil pomp
;  LEN  n 5
;  6000 direction of sprites

  org 4096-64

; left face
;SC_LF = 96
;  .byte $00     ; 00000000 
;  .byte $7C     ; 01111100
;  .byte $CE     ; 11001110
;  .byte $07     ; 00000111
;  .byte $07     ; 00000111
;  .byte $CE     ; 11001110
;  .byte $7C     ; 01111100
;  .byte $00     ; 00000000
  
; down face  
;SC_DF = 97  
;  .byte $18     ; 00011000
;  .byte $3C     ; 00111100
;  .byte $7E     ; 01111110
;  .byte $66     ; 01100110
;  .byte $42     ; 01000010
;  .byte $42     ; 01000010
;  .byte $66     ; 01100110
;  .byte $24     ; 00100100
  
; right face  
;SC_RF = 98
;  .byte $00     ; 00000000
;  .byte $2E     ; 00111110   
;  .byte $73     ; 01110011
;  .byte $E0     ; 11100000
;  .byte $E0     ; 11100000 
;  .byte $73     ; 01110011
;  .byte $2E     ; 00111110       
;  .byte $00     ; 00000000
  
; oil to peak-up
SC_OIL = 46;99
;  .byte $00     ; 00000000 
;  .byte $00     ; 00000000
;  .byte $3E     ; 00111100    
;  .byte $3E     ; 00111100
;  .byte $3E     ; 00111100  
;  .byte $3E     ; 00111100  
;  .byte $00     ; 00000000
;  .byte $00     ; 00000000
  
; monster sprite  
  .byte 192,12,0,107,88,0,31,224,0
  .byte 19,32,0,63,240,0,63,240,0
  .byte 103,152,0,195,12,0,0,0,0
  .byte 0,0,0,0,0,0,0,0,0
  .byte 0,0,0,0,0,0,0,0,0
  .byte 0,0,0,0,0,0,0,0,0
  .byte 0,0,0,0,0,0,0,0,0,0
    
  org 4096
  
init:
      lda  #0                     ; game is in pause  
      sta  GO                    
                                  ; install irq routine
      sei
      lda  #<irqR
      sta  $0314
      lda  #>irqR
      sta  $0315
      cli

      lda  #15
      sta  54272+24       ; init volume      
                                  ; oscillator 3 for random number   
      ;lda  #$ff
      ;sta  $d416
      ;lda  #$03
      ;sta  $d417
      lda  #129
      sta  54272+18
      sta  54272+14
      sta  54273+14         
         
      lda  #$30
      sta  POSX+1                 ; position x to $30xx
      lda  #$40
      sta  POSY+1                 ; position y to $40xx
      lda  #$50
      sta  DIRS+1                 ; stored dir to $50xx          

            
      lda  #MAXDELAY              ; set frame delay
      sta  DELAY                           
                                  ; insert sprite pointer 
      ldy  #5
      ldx  #2
loopSp:  
      lda  #63
      sta  2040,y
      txa
      inx
      sta  SP0_COL,y              ; insert a color for sprite
      dey
      bpl  loopSp
      
      ldy  #10
      lda  #106                   ; position y of sprite 0      
loopSpy:
      sta  SP0Y,y                 ; insert sprite y position 
      clc
      adc  #24       
      dey
      dey
      bpl  loopSpy      
      
      lda #$3F
      sta SPENABLE                ; enable all the used sprites
  
MainLoop:  
      jsr  SCREEN                 ; clear the screen
      
                                  ; fill the last 18 row with green random peaces
                                                                                              
      ldx  #23                    ; last line is not filled
loopRow:                                    
      ldy  #39
loopCol:      
      jsr  SET_COOR
      
      lda  #SC_GREEN              ; put green color
      jsr  PUT_CHAR    
      
      ;lda  RANDOM                 ; put random earth peaces
      ;and  #$01
      ;clc
      ;adc  #SC_EA
      
      lda  #SC_EA1
      jsr  PUT_CHAR                                  
      
      dey
      bpl  loopCol
      dex
      cpx  #5
      bne  loopRow
      
                                  ; put line 8, 11, 14, 17, 20, 23 with peaces to pich-up
      ldx  #7
loopRow1:
      ldy  #39
loopCol1:
      tya                         ; this push and follow pop are necessary for line 11
      pha
      
      jsr  SET_COOR
      
      lda  #SC_WHITE
      jsr  PUT_CHAR
      
      lda  #SC_OIL
      jsr  PUT_CHAR
      
      pla
      tay      
      
      dey
      bpl  loopCol1
      
      inx
      inx
      inx
      cpx  #25
      bne  loopRow1       
      
                                   ; complete with rigth vertical tunnels                                 
      lda  #SP_SPACE                                    
      sta  1024+8*40+10
      sta  1024+9*40+10
      sta  1024+8*40+30
      sta  1024+9*40+30         
      
      sta  1024+11*40+4
      sta  1024+12*40+4
      sta  1024+11*40+26
      sta  1024+12*40+26
      
      sta  1024+14*40+7
      sta  1024+15*40+7
      sta  1024+14*40+33
      sta  1024+15*40+33    
      
      sta  1024+17*40+12
      sta  1024+18*40+12
      sta  1024+17*40+22
      sta  1024+18*40+22
      
      sta  1024+20*40+9
      sta  1024+21*40+9
      sta  1024+20*40+29
      sta  1024+21*40+29                                         
      
      lda  #20                    ; initial position of oil pomp
      sta  $3000
      lda  #7
      sta  $4000
        
      lda  #93                    ; put oil start 
      sta  1024+6*40+20
      lda  #1                     ; white color
      sta  55296+6*40+20
      sta  55296+5*40+20
      ;sta  55296+4*40+20            
      lda  #81
      sta  1024+7*40+20           ; put oil head         
      lda  #127               
      sta  1024+5*40+20           ; put oil tower
      ;sta  1024+4*40+20
      
      ldx  #1
      ldy  #5
      jsr  SET_COOR
      lda  #<Sharas
      ldy  #>Sharas
      jsr  STRING       
      
      lda  #$00
      sta  BACK_COL0
      sta  BORDER_COL0     
      
      sta  POSX  
      sta  POSY
      sta  DIRS
      sta  LEN                    ; len is 0
      sta  DIR                    ; dir=dn
      sta  $5000                  ; initial dir dn
      sta  TAKE     
                                  ; insert initial random direction and monster position
      ldy  #5
loopInit:      
      tya
      asl
      tax
      lda  RANDOM
      sta  SP0X,x                 ; insert random position  
      
      lda  RANDOM               
      and  #$01
      sta  $6000,y                ; insert random direction 
                 
      dey
      bpl  loopInit                 
      
      
      inc  GO                     ; game can start
      
loop:      
      lda  TAKE
      cmp  #MAX_OIL
      beq  win
      
      lda  GO
      beq  died
      
      jmp  loop
      
win:
      dec  GO
                                  ; write winning string
      ldx  #1
      ldy  #5
      jsr  SET_COOR
      lda  #<Swin
      ldy  #>Swin
      jsr  STRING       
    
startFire:
      lda  #5                    ; write fire screen with white color
      sta  Sfire
      jsr  writeFire  
      
repeatF:                         ; attend a fire press action
      lda  $DC01
      and  #16
      bne  repeatF 

      lda  #144                  ; write fire screen with black color = erase previous
      sta  Sfire
      jsr  writeFire      
      
      jmp  MainLoop               ; start a new game
      
died:
      ldx  #1
      ldy  #5
      jsr  SET_COOR
      lda  #<Sover
      ldy  #>Sover
      jsr  STRING             

      jmp  startFire
      
; write press fire screen
writeFire:
      ; write fire string
      ldx  #1
      ldy  #27
      jsr  SET_COOR
      lda  #<Sfire
      ldy  #>Sfire
      jsr  STRING 
      rts      

; interrupt routine
irqR:
      lda  GO
      beq  exitIrq

      dec  DELAY
      bne  exitIrq

      lda  #MAXDELAY
      sta  DELAY

      jsr  userMove    
      jsr  monsterMove
exitIrq:
      jmp  $EA7D                  ; exit with ack interrupt


isDn:
      lda  (POSX),y               ; skip if we are at left of screen   
      pha
      lda  (POSY),y
      tax
      inx                         ; next row  
      pla
      tay
      pha                         ; store y (=x) position for late use

      jsr  SET_COOR
      
      lda  ($D1),y                ; read the char on where we go      
      cmp  #SP_SPACE     
      beq  notExitDn
      
      cmp  #SP_OIL                ; test for an oil to pick-up
      beq  incDn
      
      pla
      rts
      
incDn:      
      inc  TAKE                   ; we take an oil
notExitDn:      
      
      lda  #113                   ; put head
      jsr  PUT_CHAR    
    
      lda  DIR                    ; dir=0 -> dn
      beq  putCD
      bmi  putCDR
      
      lda  #176                   ; right to down corner
      bne  putCD_
      
putCDR:
      lda  #174                   ; left to down corner  
      bne  putCD_      
      
putCD: 
      lda  #125                   ; '|'
      
putCD_:       
      sta  TEMP      
      
      ldy  LEN                   
      iny
      sty  LEN 
      txa                         ; store actual position x/y  
      sta  (POSY),y
      pla
      sta  (POSX),y
      
      tay                         ; put the body of the oil pomp        
      dex
      jsr  SET_COOR
      lda  TEMP                   ; use stored value
      jsr  PUT_CHAR             
            
      lda  #0                     ; dir is now down 
      sta  DIR
      ldy  LEN
      sta  (DIRS),y               ; store actual dir   
      jsr  stepSound  
                    
exitDn:      
      rts  
      
      
; look for a move from the user
userMove:
      ldy  LEN
                                  ; look for direction
      lda  $DC01
      eor  #$FF
      sta  TEMP

      lda  #$02                   ; test for dn            
      bit  TEMP
      bne  isDn            
notDn:                            ; test for direct. sx
      asl
      bit  TEMP
      bne  isSx
notSx:                            ; test for direction dx 
      asl
      bit  TEMP
      bne  isDx
notDx:                            ; test for fire            
      asl
      bit  TEMP
      beq  notFire
      
fire:      
      lda  #2                     ; repeat 2 times for having more ... 
      sta  TEMP                   ; velocity   
      
fire_:      
      ldy  LEN                     
      beq  exitFire               ; do nothing if it is the last
      
      jsr  read_xy                ; read x,y coor of actual head
      
      jsr  SET_COOR
      lda  #32
      jsr  PUT_CHAR
      
      ldy  LEN
      dey
      sty  LEN
      lda  (DIRS),y               ; reload stored dir for this position
      sta  DIR      
            
      jsr  read_xy                ; read x,y coor of actual head
            
      jsr  SET_COOR
      lda  #113
      jsr  PUT_CHAR      
      
      dec  TEMP                   ; loop
      bne  fire_
      
      jsr  fireSound
      
exitFire:    
notFire:
exitRTS:  
      rts             
     
isSx:
      lda  (POSX),y               ; skip if we are at left of screen
      beq  exitRTS
     
      pha
      lda  (POSY),y
      tax
      pla
      tay
      dey
      tya
      pha                         ; store y (=x) position for late use

      jsr  SET_COOR
      
      lda  ($D1),y                ; read the char on where we go
      cmp  #SC_OIL                ; is a '.'
      beq  incSx
      
      cmp  #SP_SPACE              ; is a ' '
      beq  makeSx
      
      pla
      rts
                                  ; increment the picked oil
incSx:      
      inc  TAKE
      
makeSx:      
      lda  #113                   ; put head
      jsr  PUT_CHAR
      
      lda  DIR
      beq  putCL
      
      lda  #96                    ; '-'  
      bne  putCL_
            
putCL: 
      lda  #189                   ; up to left corner
putCL_:      
      jsr  PUT_CHAR         
      
      jsr  GET_COOR 
           
      ldy  LEN
      iny
      sty  LEN
      txa
      sta  (POSY),y
      pla
      sta  (POSX),y
      inc  DIR                    ; dir is now left
      lda  DIR
      sta  (DIRS),y               ; store actual dir
      jsr  stepSound
      rts    

isDx: 
      lda  (POSX),y               ; skip if we are at right of screen
      cmp  #39
      beq  exitRTS               
      
      pha
      lda  (POSY),y
      tax
      pla
      tay
      iny
      tya
      pha                         ; store y (=x) position for late use

      jsr  SET_COOR
      
      lda  ($D1),y                ; read the char on where we go
      cmp  #SC_OIL                ; is a '.'
      beq  incDx
      
      cmp  #SP_SPACE              ; is a ' '
      beq  makeDx
      
      pla
      rts
                                  ; increment the picked oil
incDx:      
      inc  TAKE
      
makeDx:      
      lda  #113                   ; put the head
      jsr  PUT_CHAR
      lda  #157                   ; go at sx (this is necessary for the last char of row)
      jsr  PUT_CHAR
      jsr  GET_COOR
      
      lda  DIR                    ; dir=0 -> dn
      beq  putCR
      
      lda  #96                    ; '-'  
      bne  putCR_
      
putCR: 
      lda  #173                   ; up to rigth corner
putCR_:       
      sta  TEMP    
           
      ldy  LEN                   
      iny
      sty  LEN 
      txa                         ; store actual position x/y  
      sta  (POSY),y
      pla
      sta  (POSX),y
            
      tay                         ; put the body of the oil pomp        
      dey
      jsr  SET_COOR
      lda  TEMP                   ; use stored value
      jsr  PUT_CHAR             
      
      ldy  LEN
      dec  DIR                    ; dir is now right
      lda  DIR
      sta  (DIRS),y               ; store actual dir
      jsr  stepSound
      rts       
        
      
; read pos x,y of acutal head
; Y must be already setted
read_xy:             
      lda  (POSY),y
      tax
      lda  (POSX),y
      tay      
      rts

; y=number of sprite to move      
moveMob:
      tya
      tax
      pha
      asl
      tay
      
      lda  $7000,x                ; read delay to apply
      beq  contMov                  
      
      dec  $7000,x                ; dec delay...
      jmp  skipInv                ; and exit
      
contMov:      
      lda  $6000,x                ; read direction: 0=right, 1=left
      bne  goLeft

      lda  SP0X,y
      clc
      adc  #STEP
      sta  SP0X,y
      cmp  #252
      bcc  skipInv 
      
      inc  $6000,x                ; direction=left         
skipInv:

      pla
      tay
      rts
      
goLeft:
      lda  SP0X,y
      sec
      sbc  #STEP
      sta  SP0X,y                
      cmp  #$07      
      bcs  skipInv      

      lda  RANDOM                 ; generate a random delay                
      sta  $7000,x 
       
      dec  $6000,x                ; direction=right         
      beq  skipInv
      ;jmp skipInv
      
; test collision      
testColl:
      tya
      pha
      asl
      tay
            
      lda  SP0Y,y                  
      sec
      sbc  #$32
      lsr
      lsr
      lsr
      tax
      
            
      lda  SP0X,y
      cmp  #$18
      bcc  skipTest
      
      sec
      sbc  #$18
      lsr
      lsr
      lsr
      tay
      
      jsr  SET_COOR
      lda  ($D1),y                ; read the char on where the monster is
      
      cmp  #81                    ; is the head of oil pomp
      beq  monDied                ; monster died 
      
      cmp  #SC_OIL                
      beq  skipTest
      
      cmp  #SP_SPACE
      beq  skipTest
                                  ; now we are died
      dec  GO                     ; game is stopped                                    
                        
      
skipTest:      
      pla
      tay      
      rts

; monster is died            
monDied:
      pla
      tay
      asl
      tax
      
      lda  RANDOM                 ; choose a random delay  
      sta  $7000,y
      
      lda  #8
      sta  SP0X,x                 ; set x coord to 8 
      rts
            
; make the movement of all the monster      
monsterMove:
      ldy  #5
loop1:   
      jsr  testColl                ; test collision first time
      jsr  moveMob                 ; move monsters
      jsr  testColl                ; test collision second time (just for more reliability)
      
      dey
      bpl  loop1
      
      rts
      
; made step sound        
stepSound:                                           
      lda  #$80                    ; release gate
      sta  54272+11

      lda  #$11                    ; triangle
      sta  54272+11
      lda  #01
      sta  54272+12

      lda  #100
      sta  54272+7
      sta  54272+8
      rts
      
fireSound:                              ; made fall down sound
      lda  #$80               ; release gate
      sta  54272+4

      lda  #$81               ; noise
      sta  54272+4
      lda  #01
      sta  54272+5

      lda  #117
      sta  54272
      sta  54272+1      
      rts
      
      
Sharas: .byte 5,"HARAS    ",0
Swin:   .byte 5,"YOU WIN! ",0
Sover:  .byte 5,"GAME OVER",0      
Sfire:  .byte 5,"PRESS FIRE",0      
      
;      bcc  noIncD
      
;      txa
;      tay
;      lda  #1
;loopD:      
;      cpy  #0
;      beq  goIncD
;      dey
;      asl                          ; shift the 1
;      jmp  loopD
      
;goIncD:
;      ora  SPX                     ; make the msb of sprite pos x high     
;      sta  SPX     
      
;noIncD:        
